# -*- coding: iso-8859-1 -*-
# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

# $Id: products.py,v 1.30 2005/06/29 15:12:49 mcourtoi Exp $
# $Name:  $

"""This module defines SETUP instances for each products.

Functions are named : setup_`product`,
and have two main arguments : DEPENDENCIES and SUMMARY objects,
and additionnal arguments through 'kargs' :
   test            : just call PreCheck if True.
"""
# Of course the order is not important... but it's easier to understand
# dependencies in the right order !

import sys
import os
import os.path
import shutil
import re
from as_setup import *

#-------------------------------------------------------------------------------
# 10. ----- hdf5
def setup_hdf5(dep, summary, **kargs):
   cfg=dep.cfg
   product='hdf5'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT',],
      set=['HOME_HDF',],
   )
   cfg['HOME_HDF']=os.path.join(cfg['ASTER_ROOT'],'public','hdf5-1.4.4')

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='1.4.4',
      description="""HDF5 is a Hierarchical Data Format product consisting of a data format
   specification and a supporting library implementation. HDF5 is designed to
   address some of the limitations of the older HDF product and to address current
   and anticipated requirements of modern systems and applications.""",
      archive='hdf5-1.4.4.tar.gz',
      content='hdf5-1.4.4',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Configure', {}),
         ('Make'     , {}),
         ('Install'  , {}),
         ('Clean'    , {}),
      ),

      installdir  = cfg['HOME_HDF'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 12. ----- med
def setup_med(dep, summary, **kargs):
   cfg=dep.cfg
   product='med'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'HOME_HDF',],
      set=['HOME_MED',],
   )
   cfg['HOME_MED']=os.path.join(cfg['ASTER_ROOT'],'public','med2.2.2')
   os.environ['HDF5HOME']=cfg['HOME_HDF']

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='2.2.2',
      description="""MED (Modele d'change de donnes) vise  fournir une passerelle
   logicielles  base de modules externes rutilisables par diffrents couples
   de codes.""",
      archive='med_fichier2.2.2.src.tar.gz',
      content='med_fichier2.2.2.src',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Configure', {}),
         ('Make'     , {}),
         ('Install'  , {}),
         ('Clean'    , {}),
      ),

      installdir  = cfg['HOME_MED'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 13. ----- tcl
def setup_tcl(dep, summary, **kargs):
   cfg=dep.cfg
   product='tcl'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT',],
      set=['HOME_TCL_TK',],
   )
   cfg['HOME_TCL_TK']=os.path.join(cfg['ASTER_ROOT'],'public')

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='8.4.5',
      description="""Tcl provides a powerful platform for creating integration applications that
   tie together diverse applications, protocols, devices, and frameworks.
   When paired with the Tk toolkit, Tcl provides the fastest and most powerful
   way to create GUI applications that run on PCs, Unix, and the Macintosh.
   Tcl can also be used for a variety of web-related tasks and for creating
   powerful command languages for applications.""",
      archive='tcl-src-8.4.5-1.tar.gz',
      content='tcl8.4.5/unix',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Configure', {}),
         ('Make'     , {}),
         ('Install'  , {}),
      ),

      installdir  = cfg['HOME_TCL_TK'],
      workdir     = '/tmp/install_tcltk.'+str(os.getpid()),
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
#    if not cfg.get('_install_tk', 'tk' in to_install):
#       setup.Clean(to_delete=setup.workdir)
   return setup

#-------------------------------------------------------------------------------
# 14. ----- tk
def setup_tk(dep, summary, **kargs):
   cfg=dep.cfg
   product='tk'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'HOME_TCL_TK', 'INCLUDEX11'],
      set=['WISH_EXE', 'INCLUDEX11',],
   )
   cfg['WISH_EXE']=os.path.join(cfg['HOME_TCL_TK'],'bin','wish8.4')

   # ----- check for X11 headers
   ftools=kargs['find_tools']
   ftools.find_and_set(cfg, 'INCLUDEX11', 'X.h', kargs['incdirs'], typ='inc',err=False)

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='8.4.5',
      description="""This directory contains the sources and documentation for Tk, an X11
   toolkit implemented with the Tcl scripting language.""",
      archive='tk-src-8.4.5-1.tar.gz',
      content='tk8.4.5/unix',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Configure', {}),
         ('Make'     , {}),
         ('Install'  , {}),
         ('Clean'    , {}),
      ),

      installdir  = cfg['HOME_TCL_TK'],
      workdir     = '/tmp/install_tcltk.'+str(os.getpid()),
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 15. ----- Numeric
def setup_Numeric(dep, summary, **kargs):
   cfg=dep.cfg
   product='Numeric'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['SITE_PACKAGES'],
   )
   # ----- setup instance
   setup=SETUP(
      product=product,
      version='23.0',
      description="""Numeric module defining a multi-dimensional array and useful procedures for
   Numerical computation.""",
      archive='Numeric-23.0.tar.gz',
      content='Numeric-23.0',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Install'  , { 'command' : sys.executable+' setup.py install'}),
      ),

      installdir  = cfg['SITE_PACKAGES'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 16. ----- Pmw
def setup_Pmw(dep, summary, **kargs):
   cfg=dep.cfg
   product='Pmw'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['SITE_PACKAGES'],
   )
   # ----- setup instance
   setup=SETUP(
      product=product,
      version='1.2',
      description="""Pmw is a Tk widgets toolkit for Python.""",
      archive='Pmw-1.2-1.tar.gz',
      content='.',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
      ),

      installdir  = cfg['SITE_PACKAGES'],
      workdir     = cfg['SITE_PACKAGES'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 20. ----- gmsh
def setup_gmsh(dep, summary, **kargs):
   cfg=dep.cfg
   product='gmsh'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR',],
   )

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='1.60.1',
      description="""Gmsh is an automatic three-dimensional finite element mesh generator,
   primarily Delaunay, with built-in pre- and post-processing
   facilities. Its primal design goal is to provide a simple meshing tool
   for academic test cases with parametric input and up to date
   visualization capabilities.  One of the strengths of Gmsh is its
   ability to respect a characteristic length field for the generation of
   adapted meshes on lines, surfaces and volumes.""",
      archive='gmsh-1.60.1-Linux.tgz',
      content='gmsh-1.60.1',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Install'  , { 'command' : 'cd '+cfg['TOOLS_DIR']+' ; rm -f gmsh ; ln -s '+\
            os.path.join(os.path.abspath(cfg['TOOLS_DIR']),'gmsh-1.60.1','gmsh')+' .' }),
      ),

      installdir  = cfg['TOOLS_DIR'],
      workdir     = cfg['TOOLS_DIR'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 21. ----- gibi
def setup_gibi(dep, summary, **kargs):
   cfg=dep.cfg
   product='gibi'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR', 'PYTHON_EXE'],
      set=['HOME_GIBI']
   )
   cfg['HOME_GIBI']=os.path.join(cfg['TOOLS_DIR'], 'gibi-2000')

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='2000',
      description="""Gibi is a scriptable three dimensional mesh generator and
   a post-processing viewer.""",
      archive='gibi-2000-3.i386.tar.gz',
      content='gibi-2000',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('ChgFiles' , {
            'files'     : ['gibi_aster.py'],
            'path'      : os.path.join(cfg['TOOLS_DIR'], 'gibi-2000'),
            'dtrans'    : cfg,
         }),
         ('Chmod'    , {
            'files'     : ['gibi_aster.py', 'gibi2000', 'gibiPC_Linux_2000'],
            'path'      : os.path.join(cfg['TOOLS_DIR'], 'gibi-2000'),
            'mode'      : 0755,
         }),
         ('Chmod'    , {
            'files'     : ['USRDAT',],
            'path'      : os.path.join(cfg['TOOLS_DIR'], 'gibi-2000'),
            'mode'      : 0666,
         }),
         ('Install'  , { 'command' :
            'cd '+cfg['TOOLS_DIR']+' ; rm -f gibi gibi.x gibi_rest ; '+ \
            'ln -s '+os.path.join(os.path.abspath(cfg['TOOLS_DIR']),'gibi-2000','gibi_aster.py')+' gibi ;'+ \
            'ln -s gibi gibi.x ; ln -s gibi gibi_rest'
         }),
      ),

      installdir  = cfg['TOOLS_DIR'],
      workdir     = cfg['TOOLS_DIR'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 30. ----- astk
def setup_astk(dep, summary, **kargs):
   cfg=dep.cfg
   product='astk'

   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'ASTER_VERSION', 'HOME_ASTK', 'TOOLS_DIR',
           'HOME_PYTHON', 'PYTHON_EXE', 'IFDEF', 'MAXCMDLEN',
           'XTERM', 'EDITOR', 'SHELL_EXECUTION', 'WISH_EXE',
           'SERVER_NAME', 'DOMAIN_NAME', 'FULL_SERVER_NAME', 'NODE' ],
      set=['HOME_TCL_TK', 'WISH_EXE',],
   )

   if cfg.has_key('HOME_TCL_TK'):
      paths=[os.path.join(cfg['HOME_TCL_TK'],'bin'),]
   else:
      paths=[]
   # should work with most of these versions (note empty string '')
   # (8.5 never tested : at the end)
   ftools=kargs['find_tools']
   ftools.find_and_set(cfg, 'WISH_EXE',
      filenames=['wish'+v for v in ['8.4', '84', '8.3', '83', '', '8.5', '85',]],
      std_dirs=kargs['bindirs'],
      paths=paths)
   if not cfg.has_key('HOME_TCL_TK'):
      cfg['HOME_TCL_TK']=os.path.abspath(os.path.join(cfg['WISH_EXE'],os.pardir,os.pardir))

   # specific values for 'ASTK_SERV' files
   astk_cfg=cfg.copy()
   astk_cfg[cfg['IFDEF']]='\n'
   astk_cfg['vers : VVV']='?vers : VVV?\nvers : %s' % cfg['ASTER_VERSION']
   # patch for zsh in as_serv
   if re.search('zsh$', astk_cfg['SHELL_EXECUTION']):
      astk_cfg['USE_ZSH']=' added for zsh\n'
   if astk_cfg.has_key('OPT_ENV'):
      astk_cfg['OPT_ENV']='\n'+astk_cfg['OPT_ENV']

   log=kargs['log']
   def astkInstall(self, **kargs):
      """'make install' method for astk
      """
      prev=os.getcwd()
      log._print(self._fmt_enter % self.installdir)
      os.chdir(self.installdir)

      for d in ('ASTK_CLIENT', 'ASTK_SERV'):
         if os.path.exists(d+'.old'):
            shutil.rmtree(d+'.old')
         if os.path.exists(d):
            os.rename(d, d+'.old')
         os.rename(os.path.join(self.workdir,self.content, d), d)

      log._print(self._fmt_leave % self.installdir)
      os.chdir(prev)

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='1.2.4',
      description='ASTK is the Graphical User Interface to manage Code_Aster calculations.',
      archive='astk-1.2.4-1.noarch.tar.gz',
      content='astk-1.2.4',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('ChgFiles' , {
            'files'     : ['ASTK_CLIENT/bin/astk', 'ASTK_CLIENT/bin/bsf',
                           'ASTK_CLIENT/lib/ASTK/astkrc/*',
                           'ASTK_SERV/bin/*', 'ASTK_SERV/lib/sub_script',
                           'ASTK_SERV/conf/*', ],
            'dtrans'    : astk_cfg,
         }),
         ('Install'  , { 'external' : astkInstall }),
         ('PyCompile', { 'path' : os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','lib') }),
         ('Chmod'    , {
            'files'     : ['ASTK_CLIENT/bin/astk', 'ASTK_CLIENT/bin/bsf',
                           'ASTK_SERV/bin/*', 'ASTK_SERV/lib/sub_script',
                           'ASTK_SERV/conf/aster_profile.sh' ],
            'mode'      : 0755,
         }),
         ('Clean'    , {}),
      ),

      installdir  = cfg['HOME_ASTK'],
      workdir     = cfg['HOME_ASTK'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 31. ----- astk : server part only
def setup_astk_serv(dep, summary, **kargs):
   cfg=dep.cfg
   product='astk_serv'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'ASTER_VERSION', 'HOME_ASTK',
           'HOME_PYTHON', 'PYTHON_EXE', 'IFDEF', 'MAXCMDLEN',
           'EDITOR', 'SHELL_EXECUTION','NODE', ],
   )

   # specific values for 'ASTK_SERV' files
   astk_cfg=cfg.copy()
   astk_cfg[cfg['IFDEF']]='\n'
   astk_cfg['vers : VVV']='?vers : VVV?\nvers : %s' % cfg['ASTER_VERSION']
   # patch for zsh in as_serv
   if re.search('zsh$', astk_cfg['SHELL_EXECUTION']):
      astk_cfg['USE_ZSH']=' added for zsh\n'
   if astk_cfg.has_key('OPT_ENV'):
      astk_cfg['OPT_ENV']='\n'+astk_cfg['OPT_ENV']

   log=kargs['log']
   def astk_servInstall(self, **kargs):
      """'make install' method for astk
      """
      prev=os.getcwd()
      log._print(self._fmt_enter % self.installdir)
      os.chdir(self.installdir)

      for d in ('ASTK_SERV',):
         if os.path.exists(d+'.old'):
            shutil.rmtree(d+'.old')
         if os.path.exists(d):
            os.rename(d, d+'.old')
         os.rename(os.path.join(self.workdir,self.content, d), d)

      log._print(self._fmt_leave % self.installdir)
      os.chdir(prev)

   # ----- astk setup instance
   setup=SETUP(
      product=product,
      version='1.2.4',
      description="""ASTK is the Graphical User Interface to manage Code_Aster calculations,
   only server part is installed now.""",
      archive='astk_serv-1.2.4-1.noarch.tar.gz',
      content='astk-1.2.4',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('ChgFiles' , {
            'files'     : ['ASTK_SERV/bin/*', 'ASTK_SERV/lib/sub_script',
                           'ASTK_SERV/conf/*', ],
            'dtrans'    : astk_cfg,
         }),
         ('Install'  , { 'external' : astk_servInstall }),
         ('PyCompile', { 'path' : os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','lib') }),
         ('Chmod'    , {
            'files'     : ['ASTK_SERV/bin/*', 'ASTK_SERV/lib/sub_script',
                           'ASTK_SERV/conf/aster_profile.sh' ],
            'mode'      : 0755,
         }),
         ('Clean'    , {}),
      ),

      installdir  = cfg['HOME_ASTK'],
      workdir     = cfg['HOME_ASTK'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 40. ----- Metis for Code_Aster
def setup_metis(dep, summary, **kargs):
   cfg=dep.cfg
   product='metis-edf'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['TOOLS_DIR',],
   )

   log=kargs['log']
   def metisInstall(self, **kargs):
      """'make install' method for metis
      """
      prev=os.getcwd()
      log._print(self._fmt_enter % self.installdir)
      os.chdir(self.installdir)

      for f in ('onmetis', 'pmetis', 'kmetis'):
         if os.path.exists(f):
            os.remove(f)
         os.symlink(os.path.abspath(os.path.join(self.content,f)), f)

      log._print(self._fmt_leave % self.installdir)
      os.chdir(prev)

   # ----- setup instance
   setup=SETUP(
      product='metis-edf',
      version='4.0',
      description="""METIS is a software package for partitioning unstructured graphs,
   partitioning meshes, and computing fill-reducing orderings of sparse matrices.""",
      archive='metis-edf-src-4.0-1.noarch.tar.gz',
      content='metis-edf-4.0',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Make'     , {}),
         ('Install'  , { 'external' : metisInstall }),
         ('Chmod'    , {
            'path'      : os.path.join(cfg['TOOLS_DIR'],'metis-edf-4.0'),
            'files'     : ['onmetis', 'onmetis.exe', 'pmetis', 'kmetis'],
            'mode'      : 0755,
         }),
      ),

      installdir  = cfg['TOOLS_DIR'],
      workdir     = cfg['TOOLS_DIR'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 41. ----- homard
def setup_homard(dep, summary, **kargs):
   cfg=dep.cfg
   product='homard'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR',],
   )
   if not os.path.isdir(os.path.join(cfg['TOOLS_DIR'],'HOMARD')):
      yes='y\n'
   else:
      yes=''

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='7.1',
      description="""The HOMARD software carries out the adaptation of 2D/3D finite element or
   finite volume meshes by refinement and unrefinement techniques.""",
      archive='homard-7.1-1.i386.tar.gz',
      content='HOMARD',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Install'  , { 'command' :
            'echo "%(inst)s\n%(rep)s%(py)s\n" | python setup_homard.py' % \
            { 'inst' : os.path.join(cfg['TOOLS_DIR'],'HOMARD'),
              'rep'  : yes,
              'py'   : cfg['PYTHON_EXE'] } }),
         ('Install'  , { 'command' : 'cd '+cfg['TOOLS_DIR']+' ; rm -f homard ; ln -s '+\
            os.path.join(os.path.abspath(cfg['TOOLS_DIR']),'HOMARD','homard')+' .' }),
         ('Clean'    , {}),
      ),

      installdir  = cfg['TOOLS_DIR'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 42. ----- eficas
def setup_eficas(dep, summary, **kargs):
   cfg=dep.cfg
   product='eficas'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR',],
   )

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='1.8.0',
      description="""Eficas is an graphical editor for Code_Aster command files.""",
      archive='eficas-1.8.0-1.tar.gz',
      content='eficas-1.8.0',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('Install'  , { 'command' :
            """rm -f %(script)s ; echo '#!/bin/sh\n%(py)s %(inst)s/Aster/eficas_aster.py $*\n' > %(script)s""" % \
            { 'inst' : os.path.abspath(os.path.join(cfg['TOOLS_DIR'],'eficas-1.8.0')),
              'script' : os.path.abspath(os.path.join(cfg['TOOLS_DIR'],'eficas')),
              'py'   : cfg['PYTHON_EXE'] } }),
         ('Chmod' , {
            'files'     : ['eficas'],
            'path'      : cfg['TOOLS_DIR'],
            'mode'      : 0755,
         }),
         ('PyCompile', { 'path' :
            os.path.join(cfg['TOOLS_DIR'],'eficas-1.8.0'), }),
      ),

      installdir  = cfg['TOOLS_DIR'],
      workdir     = cfg['TOOLS_DIR'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 50. ----- Code_Aster
def setup_aster(dep, summary, **kargs):
   cfg=dep.cfg
   product='aster'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR', 'ASTER_VERSION', 'HAVE_F90',
           'HOME_PYTHON', 'PYTHON_EXE', 'PYTHONLIB',
           'HOME_MUMPS', 'HOME_ZMAT', 'HOME_MPI',
           'HOME_MED', 'HOME_HDF', 'HOME_CRPCRS',
           'LD', 'CC', 'F77', 'F90',],
      reqobj=['file:?HOME_ASTK?/ASTK_SERV/bin/as_run',
              'file:?HOME_ASTK?/ASTK_SERV/conf/aster_profile.sh'],
      set=['SYSLIB', 'OTHERLIB',
           'MEDLIB', 'HDFLIB', 'BLASLIB', 'LAPACKLIB',
           'MUMPSLIB', 'ZMATLIB',
           'LDFLAGS',
           'CFLAGS', 'CFLAGS_DBG', 'CINCLUDE',
           'FFLAGS', 'FFLAGS_DBG', 'FINCLUDE',
           'F90FLAGS', 'F90FLAGS_DBG', 'F90INCLUDE',
           'NOBUILD', ],
   )
   cfg['OTHERLIB'] = cfg.get('OTHERLIB', '')
   cfg['NOBUILD']  = cfg.get('NOBUILD', '')
   cfg['OPT_ENV']  = cfg.get('OPT_ENV', '')
   cfg['OPT_ENV']  = '?OPT_ENV?\n\n' + cfg['OPT_ENV']
   ftools=kargs['find_tools']

   # ----- check for Python headers
   ftools.find_and_set(cfg, 'CINCLUDE', 'Python.h',
      kargs['incdirs'],
      os.path.join(cfg['HOME_PYTHON'],'include','python'+sys.version[:3]),
      typ='inc', err=True, append=True)
   # ----- check for Python/Numeric headers
   ftools.find_and_set(cfg, 'CINCLUDE', 'arrayobject.h',
      kargs['incdirs'],
      os.path.join(cfg['HOME_PYTHON'],'include','python'+sys.version[:3],'Numeric'),
      typ='inc', err=True, append=True)
   # ----- check for BLAS library
   # prefers shared libraries because of obscur dependencies blas/lapack
   ftools.findlib_and_set(cfg, 'BLASLIB', 'blas',
      kargs['libdirs'], '/usr/lib/atlas',
      typ='lib', err=True, append=False, prefshared=True)
   # ----- check for LAPACK library
   ftools.findlib_and_set(cfg, 'LAPACKLIB', 'lapack',
      kargs['libdirs'], '/usr/lib/atlas',
      typ='lib', err=True, append=False, prefshared=True)

   cxxlibs=[]
   zmat_platform=''
   mpilibs=[]
   opt={}
   if   cfg['IFDEF']=='P_LINUX':
      opt['BLASLIB']    = '-lblas'
      opt['LAPACKLIB']  = '-llapack'
      opt['SYSLIB']     = '-Xlinker --export-dynamic -lieee -ldl -lpthread -lutil -lm'
      opt['LDFLAGS']    = '-v'
      opt['CFLAGS_DBG'] = '-c -g -DP_LINUX'
      opt['CFLAGS']     = '-c -O2 -DP_LINUX'
      opt['FFLAGS_DBG'] = '-c -g'
      opt['FFLAGS']     = '-c -O2'
      opt['FINCLUDE']   = ''
      cxxlibs.extend(['stdc++', 'supc++'])
      zmat_platform='Linux4'
      mpilibs.extend(['mpich'])
   elif cfg['IFDEF']=='x86_64':
      opt['BLASLIB']    = '-lblas'
      opt['LAPACKLIB']  = '-llapack'
      opt['SYSLIB']     = '-Xlinker --export-dynamic -lieee -ldl -lpthread -lutil -lm'
      opt['LDFLAGS']    = '-v'
      opt['CFLAGS_DBG'] = '-c -g -DTRU64'
      opt['CFLAGS']     = '-c -O2 -DTRU64'
      opt['FFLAGS_DBG'] = '-c -g'
      opt['FFLAGS']     = '-c -O2'
      opt['FINCLUDE']   = ''
      cxxlibs.extend(['stdc++', 'supc++'])
      mpilibs.extend(['mpich'])
   elif cfg['IFDEF']=='TRU64':
      opt['BLASLIB']    = '-lcxml'
      opt['LAPACKLIB']  = ''
      opt['SYSLIB']     = '/usr/lib/libots3.a /usr/lib/libpthread.a /usr/lib/libnuma.a /usr/lib/libpset.a  /usr/lib/libmach.a -lUfor -lfor -lFutil -lm -lots -lm_c32 -lmld /usr/ccs/lib/cmplrs/cc/libexc.a  '
      opt['LDFLAGS']    = '-v'
      opt['CFLAGS_DBG'] = '-v -c -g  -omp -ansi_alias -ansi_args -DOMP -DTRU64'
      opt['CFLAGS']     = '-v -c -O3 -omp -ansi_alias -ansi_args -DOMP -DTRU64'
      opt['FFLAGS_DBG'] = '-v -c -g    -i8 -r8 -omp'
      opt['FFLAGS']     = '-v -c -fast -i8 -r8 -omp'
      opt['FINCLUDE']   = ''
      cxxlibs.extend(['rt', 'cxx', 'cxxstd'])
      mpilibs.extend(['mpi'])
   elif cfg['IFDEF']=='SOLARIS':
      opt['BLASLIB']    = '-lblas'
      opt['LAPACKLIB']  = '-llapack'
      opt['SYSLIB']     = '-lsocket -lnsl /usr/lib/libintl.so.1 -ldl -lc'
      opt['LDFLAGS']    = '-v'
      opt['CFLAGS_DBG'] = '-c -g -DSOLARIS'
      opt['CFLAGS']     = '-c -DSOLARIS'
      opt['FFLAGS_DBG'] = '-c -g  -stackvar -dalign'
      opt['FFLAGS']     = '-c -O3 -stackvar -xchip=ultra -xarch=v8plusa -dalign'
      opt['FINCLUDE']   = ''
   elif cfg['IFDEF']=='SOLARIS64':
      opt['BLASLIB']    = '-lsunperf'
      opt['LAPACKLIB']  = ''
      opt['SYSLIB']     = '-lsocket -lnsl -ldl -lc -lm -lF77 -lM77'
      opt['LDFLAGS']    = '-v -xarch=v9 -dalign -xtypemap=real:64,double:64,integer:64'
      opt['CFLAGS_DBG'] = '-c -g -xarch=v9 -dalign -DSOLARIS64'
      opt['CFLAGS']     = '-c -g -xarch=v9 -dalign -DSOLARIS64'
      opt['FFLAGS_DBG'] = '-c -g -stackvar -xarch=v9 -dalign -xtypemap=real:64,double:64,integer:64'
      opt['FFLAGS']     = '-c -stackvar -xarch=v9 -dalign -xtypemap=real:64,double:64,integer:64'
      opt['FINCLUDE']   = ''
   elif cfg['IFDEF']=='IRIX':
      opt['BLASLIB']    = '/usr/lib64/libcomplib.sgimath.so'
      opt['LAPACKLIB']  = ''
      opt['SYSLIB']     = '-lfpe -lm'
      opt['LDFLAGS']    = '-v -64 -mips4'
      opt['CFLAGS_DBG'] = '-c -g -64 -xansi -DIRIX -DIRIX_64'
      opt['CFLAGS']     = '-c    -64 -xansi -DIRIX -DIRIX_64'
      opt['FFLAGS_DBG'] = '-c -g  -DEBUG:trap_unitialized=ON -64 -i8 -G0'
      opt['FFLAGS']     = '-c -g3 -OPT:roundoff=0:cray_ivdep=TRUE:Olimit=0 -mips4 -64 -i8 -G0'
      opt['FINCLUDE']   = ''

   # ----- F90
   opt['F90FLAGS_DBG'] = opt['FFLAGS_DBG']
   opt['F90FLAGS']     = opt['FFLAGS']
   opt['F90INCLUDE']=''

   # ----- check for MED and HDF5 libraries, and HDF5 includes
   if cfg['HOME_HDF']<>'':
      ftools.findlib_and_set(cfg, 'HDFLIB', 'hdf5',
         kargs['libdirs'], os.path.join(cfg['HOME_HDF'],'lib'),
         typ='lib', err=True, append=False)
      ftools.find_and_set(cfg, 'CINCLUDE', 'hdf5.h',
         kargs['incdirs'],
         os.path.join(cfg['HOME_HDF'],'include'),
         typ='inc', err=True, append=True)
      # ----- check for libz library
      ftools.findlib_and_set(cfg, 'OTHERLIB', 'z',
         kargs['libdirs'],
         typ='lib', err=False, append=True)
   else:
      if cfg['NOBUILD'].find('bibc/hdf')<0:
         cfg['NOBUILD'] += ' bibc/hdf'
      cfg['HOME_MED']=''
      cfg['HDFLIB']=''

   if cfg['HOME_MED']<>'':
      ftools.findlib_and_set(cfg, 'MEDLIB', 'med',
         kargs['libdirs'], os.path.join(cfg['HOME_MED'],'lib'),
         typ='lib', err=True, append=False)
   else:
      cfg['MEDLIB']=''

   # ----- MUMPS
   if cfg['HOME_MUMPS']<>'':
      for lib in ('dmumps', 'zmumps', 'pord', 'mpiseq'):
         ftools.findlib_and_set(cfg, 'MUMPSLIB', lib,
            kargs['libdirs'], os.path.join(cfg['HOME_MUMPS'], 'lib'),
            typ='lib', err=False, append=True)
      opt['F90INCLUDE'] += ' -I%s' % os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION'],'bibf90','include_mumps')
   else:
      opt['MUMPSLIB']=''
      # if MUMPS is not available and f90 exists, don't build bibf90/mumps
      if cfg['HAVE_F90']<>'' and cfg['NOBUILD'].find('bibf90/mumps')<0:
         cfg['NOBUILD'] += ' bibf90/mumps'

   # ----- ZMAT
   if cfg['HOME_ZMAT']<>'':
      for lib in ['Zmat_base', 'Zmat_base_public', 'zAster']:
         ftools.findlib_and_set(cfg, 'ZMATLIB', lib,
            kargs['libdirs'], os.path.join(cfg['HOME_ZMAT'], 'PUBLIC'),
            typ='lib', err=False, append=True, maxdepth=max(ftools.maxdepth,1))
      for lib in cxxlibs:
         ftools.findlib_and_set(cfg, 'ZMATLIB', lib,
            kargs['libdirs'], os.path.join(cfg['HOME_ZMAT'], 'PUBLIC'),
            typ='lib', err=False, append=True, maxdepth=max(ftools.maxdepth,10))
      cfg['OPT_ENV'] += """# ZMAT environment
Z7PATH=%(HOME_ZMAT)s
export Z7PATH\n""" % cfg
      if zmat_platform<>'':
         cfg['OPT_ENV'] += """Z7MACHINE='%s'
export Z7MACHINE\n""" % zmat_platform
      cfg['OPT_ENV'] += ". $Z7PATH/lib/Z7_profile\n"

   else:
      opt['ZMATLIB']=''

   # ----- MPI only for FETI solver
   if cfg['HOME_MPI']<>'':
      opt['MPILIB']=''
      for lib in mpilibs:
         ftools.findlib_and_set(cfg, 'MPILIB', lib,
            kargs['libdirs'], os.path.join(cfg['HOME_MPI'], 'lib'),
            typ='lib', err=False, append=True)
      ftools.find_and_set(cfg, 'FINCLUDE', 'mpif.h',
         kargs['incdirs'],
         os.path.join(cfg['HOME_MPI'], 'include'),
         typ='inc', err=False, append=True)
      opt['CFLAGS']    +=' -DMPI -DMPI_FETI'
      opt['CFLAGS_DBG']+=' -DMPI -DMPI_FETI'

   else:
      opt['MPILIB']=''
      # if MPI is not available, don't build bibfor/from_c
      if cfg['NOBUILD'].find('bibfor/from_c')<0:
         cfg['NOBUILD'] += ' bibfor/from_c'

   # use previous options only if there aren't set in cfg
   for k,v in opt.items():
      if not cfg.has_key(k):  cfg[k]=v

   # add paths of *LIB variables to ftools.path['lib'] then to LD_LIBRARY_PATH
   for var in [k for k in cfg.keys() if k.endswith('LIB')]:
      ftools.addtopath('lib', cfg[var])

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='8.1',
      description="""Code_Aster finite element method solver.""",
      archive='aster-src-8.1.0-1.tar.gz',
      content='STA8.1',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , { 'extract_as' : cfg['ASTER_VERSION'] }),
         ('ChgFiles' , {
            'files'     : ['config.txt', '*.export'],
            'path'      : os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION']),
            'dtrans'    : cfg,
         }),
         ('ChgFiles' , {
            'files'     : ['aster_profile.sh'],
            'path'      : os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','conf'),
            'dtrans'    : {'LD_LIBRARY_PATH=' : 'LD_LIBRARY_PATH=%s:' % ':'.join(ftools.paths['lib']),},
            'delimiter' : '',
         }),
         ('ChgFiles' , {
            'files'     : ['aster_profile.sh'],
            'path'      : os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','conf'),
            'dtrans'    : {'OPT_ENV' : cfg['OPT_ENV'],},
         }),
         ('Make'     , {
            'path'      : os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION']),
            'command'   : 'source %s ; %s --make --version_dev=%s' % \
            (os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','conf','aster_profile.sh'),
             os.path.join(cfg['HOME_ASTK'],'ASTK_SERV','bin','as_run'),
             cfg['ASTER_VERSION']),
         }),
      ),

      installdir  = cfg['ASTER_ROOT'],
      workdir     = cfg['ASTER_ROOT'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 51. ----- Code_Aster Linux pour Alliances
def setup_aster_Linux(dep, summary, **kargs):
   cfg=dep.cfg
   product='aster_Linux'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'TOOLS_DIR', 'ASTER_VERSION',
           'HOME_PYTHON', 'PYTHON_EXE', 'PYTHONLIB',],
   )

   # ----- aster setup instance
   setup=SETUP(
      product=product,
      version='7.3.26',
      description="""Code_Aster finite element method solver, executables for Linux included.""",
      archive='aster-7.3.26-1.Linux.tar.gz',
      content='NEW7.3',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , { 'extract_as' : cfg['ASTER_VERSION'] }),
         ('ChgFiles' , {
            'files'     : ['config.txt', '*.export'],
            'path'      : os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION']),
            'dtrans'    : cfg,
         }),
         ('Chmod' , {
            'files'     : ['asteru.exe', 'asterd.exe'],
            'path'      : os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION']),
            'mode'      : 0755,
         }),
         ('PyCompile', { 'path' :
            os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION'],'bibpyt'), }),
      ),

      installdir  = cfg['ASTER_ROOT'],
      workdir     = cfg['ASTER_ROOT'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

#-------------------------------------------------------------------------------
# 60. ----- crp/crs
def setup_crpcrs(dep, summary, **kargs):
   cfg=dep.cfg
   product='crpcrs'
   # ----- add (and check) product dependencies
   dep.Add(product,
      req=['ASTER_ROOT', 'ASTER_VERSION', 'HOME_CRPCRS'],
   )
   crpcrs_cfg=cfg.copy()
   crpcrs_cfg['DABIN']     = cfg['HOME_CRPCRS']
   crpcrs_cfg['REP_ASTER'] = cfg['ASTER_ROOT']
   crpcrs_cfg['VERS']      = cfg['ASTER_VERSION']

   log=kargs['log']
   
   def crpcrsInstall(self, **kargs):
      """Prepare crp base, param and fantome directories
      """
      path=kargs['asterdir']
      dest=os.path.abspath(os.path.join(path,'crp'))
      prev=os.getcwd()
      log._print(self._fmt_enter % path)
      os.chdir(path)

      if not os.path.exists(dest):
         os.mkdir(dest)
      for d in ('base', 'fantome', 'param'):
         iret,out=self.Shell(cmd='rm -rf '+os.path.join(dest,d))
      for d in ('fantome', 'param'):
         iret,out=self.Shell(
               cmd='mv '+os.path.join(self.installdir,'CRP',d)+' '+dest)

      log._print(self._fmt_leave % path)
      os.chdir(prev)

   # ----- setup instance
   setup=SETUP(
      product=product,
      version='8.1.0',
      description="""CRP for Programmation Rules Control and CRS for Syntax Rules Control.""",
      archive='crpcrs-8.1.0-1.tar.gz',
      content='.',
      depend=dep,
      system=kargs['system'],
      log=kargs['log'],

      actions=(
         ('Extract'  , {}),
         ('ChgFiles' , {
            'files'     : ['*.sh', ],
            'dtrans'    : crpcrs_cfg,
         }),
         ('Chmod' , {
            'files'     : ['*.sh', ],
            'mode'      : 0755,
         }),
         ('Install'  , { 'external' : crpcrsInstall,
                         'asterdir' : os.path.join(cfg['ASTER_ROOT'],cfg['ASTER_VERSION']),
         }),
         ('Make'     , { 'path' :
            [ os.path.join(cfg['HOME_CRPCRS'],'CRP','SRC'),
              os.path.join(cfg['HOME_CRPCRS'],'CRP_BD','SRC'),
              os.path.join(cfg['HOME_CRPCRS'],'CRS','SRC'),
              os.path.join(cfg['HOME_CRPCRS'],'fsplit'),] }),
         ('Install'  , { 'command' : './make_base_crp.sh' }),
      ),

      installdir  = cfg['HOME_CRPCRS'],
      workdir     = cfg['HOME_CRPCRS'],
      sourcedir   = cfg['SOURCEDIR'],
   )
   if not kargs.get('test', False):
      setup.Go()
   else:
      setup.PreCheck()
   return setup

